This file will contain all of the ingredients to make up the sbt 
   eVC.  Of course, the eVC still needs to be configured to the users 
   liking in a configuration (_cfg.e) file somewhere in the users
   project directory
<'
package sbt;

import sbt_tx_agent;
import sbt_rx_agent;
import sbt_scoreboard;


//This is the sbt_environment.  It contains all of the ingredients
//that compose the sbt environment.  The tx/rx agents, the scroreboards
//as well as the reset agent are all instantiated within this unit.  This 
//unit also looks after all clocking according to one main clock.  On an
//occurence of the clock_e event, all other clocks are emitted for all 
//objects within the environment.
unit sbt_env_u like any_env {
   
   //environment can be instantiated as either ACTIVE (bfm's driving)
   //or PASSIVE (bfm's disabled)
   active_passive: erm_active_passive_t;
   
   //This clocking event shall be defined in the users
   //project directory in an extension somewhere as each 
   //user will need to tie this clock to a different hdl 
   //signal 
   event clock_e;
   
   //This field is used to indicate how many active channels there are 
   //in the environment.
   num_active_channels: uint;
   keep soft num_active_channels in range [1..3];
   
   //the DUT can only support up to a max of 10 channels
   keep num_active_channels in range [0..10];

   //Instantiate a list of TX agents in the environment
   tx_agents: list of sbt_tx_agent_u is instance;
   keep soft tx_agents.size() == num_active_channels;
   
   //Instantiate a list of RX agents in the environment
   rx_agents: list of sbt_rx_agent_u is instance;
   keep soft rx_agents.size() == num_active_channels;
   
   //The reset agent will issue a reset at the beginning of the 
   //simulation 
   reset_agent: sbt_reset_agent_u is instance;

   //This list of scoreboards (one per channel) take care of all end to 
   //end checking in the environment.  As objects are driven into the 
   //DUT, they are added to the scoreboard.  As objects are received, 
   //they are passed to the scoreboard for checking
   scoreboards: list of sbt_scoreboard_u is instance;
   keep soft scoreboards.size() == num_active_channels;
   
   keep for each in scoreboards {
      it.scoreboard_id == index+1;
   }; // keep for each i...

   keep for each in tx_agents {
      it.active_passive == active_passive;
      //push unique ID's into the agents
      it.agent_id == index+1;
   };//keep for each 
   
   keep for each in rx_agents {
      it.active_passive == active_passive;
      
      //push unique ID's into the agents
      it.agent_id == index+1;
   };//keep for each 
   
   //whenever the clocking event occurs, push the clocks
   //down into the rx and tx agents as well as the reset_agent.  
   //This may not necessarily 
   //fit everyones needs and more thought should be given towards
   //clocking 
   on clock_e {
      emit reset_agent.clock_e;
      
      for each in tx_agents { 
         emit it.clock_e;
      };//for each 
      for each in rx_agents { 
         emit it.clock_e;
      };//for each 
   }; // on clock_e
   
   
}; // unit sbt_en...

'>
