<'
package sbt;

//The sbt_monitor is used in multiple places within the environment.  The 
//tx agents use it to collect packets at the input to the DUT and the rx
//agents use it as their main colection agent. The monitor has a method
//receive_data() which collects data and passes it back to the calling method.
//There are also a number of "hook methods" for the user to extend to add 
//functionality.
unit sbt_monitor_u {

   //Since there may be more than one monitor
   //per agent, have an id field to uniquely identify
   //each object
   monitor_id: uint;
   keep soft monitor_id == 0;

   //clocking event that will be pushed in from above
   event clock_e;

   //define a string to hold the hdl data signal name
   sig_data: string;
   //define a string to hold the hdl frame pulse signal name
   sig_fp: string;

   //Create an event to indicate the rising edge of a frame
   //pulse
   event fp_rise is rise('(sig_fp)')@sim;


   //The receive data method shall read a number of bytes
   //from the DUT and then assemble a packet and return it 
   //to the calling object
   receive_data(): sbt_packet_s @clock_e is {
      
      var data_collected: list of byte;

      just_before_receiving();
      //wait for the rising edge of a frame pulse
      //before collecting data
      sync @fp_rise;
      
      //collect data while the fp signal is high
      while ('(sig_fp)' == 1'b1) {
         data_collected.add('(sig_data)');
         wait cycle;
      }; // while ('(sig_fp...

      //unpack the collected data into an sbt_packet
      //to be passed out of this method
      unpack(packing.low,data_collected, result);

      //call to a hook method
      just_after_receiving();
   };

   //This is a hook method that can be extended by 
   //the user.  It is placed at a location just after the 
   //receive_data method has been called but before any data
   //has been collected
   just_before_receiving()@clock_e is empty;

   //This is a hook method that can be extended by the user.  
   //It is placed at a location just after a ful and complete 
   //packet has been collected and unpacked into a sbt_packet_s 
   //struct.
   just_after_receiving()@clock_e is empty;

}; // unit sbt_mo...

'>
