<'
package sbt;

import sbt_types_and_defines;
import sbt_header;

//The sbt_packet is a very simple structure which consists of a start
//flag, header, payload, CRC and an end flag.  This is very similar to 
//an HDLC packet.
struct sbt_packet_s like any_sequence_item {
   
   //the start flag will typically contain the value
   //0x7E however, in this file, we will leave it unconstrained
   //such that the user can import their own set of constraints
   //into the environment to control start/end flag values
   %start_flag: byte;
   
   //instantiate a header struct 
   %header: sbt_header_s;

   //field to control the size of the payload
   //payload_size: uint;
   
   //the payload for the packet is a simple list of bytes
   %payload[80]: list of byte;
   //keep soft payload.size() == payload_size;

   //The crc_kind field is used to control the generation 
   //of both GOOD and BAD.  Values for this field are GOOD/BAD.
   crc_kind: sbt_crc_kind_t;

   //This virtual field will always contain the good 
   //crc value
   good_crc: byte;
   keep good_crc == calculate_crc();
   
   //one byte crc value will be used.  This value is constrained
   //such that, ony when the crc_kind field is GOOD will the value
   //of the crc field be correct.
   %crc: byte;
   keep crc_kind == GOOD => soft crc == good_crc;
   keep crc_kind == BAD  => soft crc != good_crc;
   
   //the end flag is unconstrained here, it 
   //will be constrained by the user elsewhere
   %end_flag: byte;
   
   //this field will act as a delay between packets
   packet_delay: uint;
   keep soft packet_delay in range [1..20];
   
   //This method should be filled in to calculate the crc over the
   //packet contents.  A generic 8 bit CRC calculation scheme will 
   //be used.  This calculation is built-into the Specman Elite tool.
   calculate_crc():byte is {
      result = payload.crc_8(0,payload.size());
   }; // calculate_crc()...

   
   //a method that will traverse through all of the contents of the 
   //packet and remove any pattern that matches the start/end flag 
   //patterns
   remove_all_embedded_flags() is empty;
   
   //extend the built-in post_generate() method (called automatically
   //by specman after every time a packet is generated) to remove all 
   //unwanted duplicate flags and also to fill in 
   //the crc field with the correct value for CRC.
   post_generate() is also {
      remove_all_embedded_flags();
      //crc = calculate_crc();
   }; // post_generate()...
   

}; // struct sbt_...

'>
