This agent is a simple reset agent that will toggle some pins high/low
   for some number of clock cycles
<'

package sbt;

//The sbt_reset_agent_u unit will assert a reset each time the 
//reset() method is called. This is an active low reset agent.
//It contains fields to control the number of clocks to hold 
//reset down for.  
unit sbt_reset_agent_u {
   
   //clocking event to be pushed in from above
   event clock_e;
   
   //This event will be emitted whenever a reset is
   //finished
   event reset_done;
   
   //define an unsigned integer to hold the number 
   //of clocks that reset should be asserted for 
   number_of_clocks: uint;
   
   //define a string to represent the reset hdl signal
   sig_reset: string;
   
   //this reset method will toggle the reset pin(s) for a directed random 
   //number of clocks
   reset()@clock_e is {
      
      //generate a new random number of clocks to hold the 
      //reset pin for.  This ensures that, if reset is ever 
      //called during a simulation, a different number of 
      //clocks will occur
      gen number_of_clocks;
      
      //active low signal
      '(sig_reset)' = 1'b0; 
      
      //wait for a random number of clocks
      wait [number_of_clocks] * cycle;
      
      //return to normal value
      '(sig_reset)' = 1'b1; 
      
      //emit the reset_done event to let the environment 
      //know that the reset is complete
      emit reset_done;

   }; // reset()@clock_e...
   
   //extend the built-in run() method to call the reset method 
   //at the start of simulation
   run() is also {
      start reset();
   }; // run() is also
   
   
 }; //

'>
