<'
package sbt;

import sbt_monitor;

//This unit wil house a monitor that will be constantly collecting
//packets from the DUT and passing them into the current_packet 
//field of this unit.  
unit sbt_rx_agent_u {
   
   //This field can be used to uniquely identify all agents within
   //a list.  When running on devices with multiple channels, this 
   //field should note the channel number
   agent_id: uint;
   keep soft agent_id == UNDEF;
   
   //clocking event to be pushed down from the environment
   event clock_e;

   //This event will be emitted each time a packet is received
   //from the monitor
   event packet_received;
   
   //This boolean will be used to disable the while loop in the 
   //receive_packets() method if needed 
   receive_packets_loop_enable: bool;
   keep soft receive_packets_loop_enable == TRUE;

   //agent can be instantiated as either ACTIVE (bfm's driving)
   //or PASSIVE (bfm's disabled)
   active_passive: erm_active_passive_t;

   //Temporary holder for the current packet that was received
   //by the monitor
   !current_packet: sbt_packet_s;

   //instantiate a list of monitors...this lists size 
   //needs to be constrained in the user environmentx
   monitor: sbt_monitor_u is instance;
   keep monitor.monitor_id == agent_id;
   
   //whenever the rx agent receives a clock, push the 
   //clock down into the monitors
   on clock_e {
      emit monitor.clock_e;
   }; // on clock_e

   //This method will continually call the monitors receive_data() 
   //method to collect and assemble packets.  The received packets
   //will be temporarily stored in the current_packet field until the 
   //next packet is collected
   receive_packets() @clock_e is {
      //loop forever, unless the boolean is set to FALSE elsewhere
      while (receive_packets_loop_enable == TRUE) {
         
         //hook method
         just_before_receiving();
         
         //Call the monitor to receive a packet
         current_packet = monitor.receive_data();
         
         //hook method
         just_after_receiving(current_packet);
         
         //emit the packet received event to let the world know 
         //that a packet was received
         emit packet_received;
      };//while
   };//receive_packets
   
   //hook method for user extension
   just_before_receiving()@clock_e is empty;
   
   //hook method for user extension
   just_after_receiving(received_packet: sbt_packet_s)@clock_e is empty;
   
   run() is also {
      start receive_packets();
   }; // run() is also

}; // unit sbt_rx...

'>
