This file creates the HDLC sequence driver and hooks the sequence driver
   into the sbt_agent_u

<'
package sbt;
 
import sbt_packet;
import sbt_tx_agent;


//use the sequence statement to create the sequence driver unit,


//the sequence struct and the enumerated type for the sequence
sequence sbt_sequence_s using item=sbt_packet_s, 
  created_driver=sbt_sequence_driver_u, 
  created_kind=sbt_sequence_kind_t;

extend sbt_sequence_s {
   //This is the basic packet item
   !packet: sbt_packet_s;
}; // extend sbt_...


//extend only the ACTIVE type of agent
extend ACTIVE sbt_tx_agent_u {
   
   //instantiate the sequence driver inside of the agent
   sequence_driver: sbt_sequence_driver_u is instance;
   
   //whenever a clock occurs in the agent, 
   //push the clock down into the sequence driver 
   on clock_e {
      emit sequence_driver.clock;
   }; // unit sbt_ag...
   
   //This is a temporary storage holder for the current 
   //item being generated and/or driven into the DUT
   !current_item: sbt_packet_s;

   //loop enable boolean field
   send_items_loop_enable: bool;
   keep soft send_items_loop_enable == TRUE;

   //this method is used to grab items from the sequence driver and
   //pass them down into the BFM for driving
   send_items_to_bfm()@clock_e is {
      
      //hook method
      just_before_looping();
      
      while (send_items_loop_enable == TRUE) do {
         current_item = sequence_driver.get_next_item();
         
         //hook method
         just_before_driving();

         //here we need to understand what the interaction is
         //between the bfm_list and the sequence driver is 
         //(two BFM's exist....what gets driven into each one?)
         bfm.drive(current_item);
   
         //hook method
         just_after_driving();

         emit sequence_driver.item_done;
      }; // while (send_ite...
      
      just_after_looping();

   }; // send_items_to_b...
   
   //hook methods for user extension to add specific functionality
   just_before_looping()@clock_e is empty;
   just_after_looping()@clock_e is empty;
   just_before_driving()@clock_e is empty;
   just_after_driving()@clock_e is empty;

   
   //extend the built-in specman run() method to 
   //start off any TCM's at the beginning of the simulation
   //run phase of a specman simulation
   run() is also {
      start send_items_to_bfm();
   }; // run() is also

}; // extend sbt_...






'>
